"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.putProxy = exports.traceProxy = exports.headProxy = exports.optionsProxy = exports.deleteProxy = exports.patchProxy = exports.postProxy = exports.getProxy = void 0;
const service_common_1 = require("@crosslab/service-common");
const node_fetch_1 = __importDefault(require("node-fetch"));
const data_source_1 = require("../data_source");
const model_1 = require("../model");
/**
 * This function implements the functionality for handling $method requests on /proxy endpoint.
 * @param method The http method to be used.
 */
const proxy = (method) => async (parameters, body, _user) => {
    if (!parameters.URL)
        throw new service_common_1.MissingParameterError(`Missing URL Parameter`, 400);
    const basePathMatch = parameters.URL.match(/.*?:\/\/.*?(?=\/|$)/gm);
    let headers = { 'Content-Type': 'application/json' };
    if (basePathMatch) {
        const basePath = basePathMatch[0];
        const InstitutionRepository = data_source_1.AppDataSource.getRepository(model_1.InstitutionModel);
        const institution = await InstitutionRepository.findOneBy({ api: basePath });
        if (institution) {
            headers['Authorization'] = 'Bearer ' + institution.apiToken;
        }
    }
    const response = await (0, node_fetch_1.default)(parameters.URL, {
        headers,
        method: method,
        body: Object.keys(body).length > 0 ? body : undefined,
    });
    if (response.status < 100 || response.status >= 600) {
        throw new service_common_1.InvalidValueError(`Response has invalid status of ${response.status}`, 500); // TODO: maybe find better error
    }
    try {
        const text = await response.text();
        return {
            status: response.status,
            body: text,
        };
    }
    catch (error) {
        console.log(error);
        return {
            status: response.status,
            body: undefined,
        };
    }
};
exports.getProxy = proxy('get');
exports.postProxy = proxy('post');
exports.patchProxy = proxy('patch');
exports.deleteProxy = proxy('delete');
exports.optionsProxy = proxy('options');
exports.headProxy = proxy('head');
exports.traceProxy = proxy('trace');
exports.putProxy = proxy('put');
//# sourceMappingURL=proxy.js.map